#!/usr/bin/perl

# "collapse.pl", a script for use with the Forum News Gateway
# copyright 1995 by Swarthmore College

# Given tab-delimited lines on standard input, extract the first
# column and lay it out on one line, with spaces between the fields.

# For example,

#	abc\tdef
#	ghi\tjkl
#	mno\tpqr

# turns into

#	abc ghi mno

# The information ends up in the right form to pass to shell commands,
# like "cat" and "rm".

# The optional prefix lets you add a directory name to each item.
# The suffix lets you treat the item as a directory, and specify a file in it.
# For example, if there exist news accounts "abc" "ghi" and "mno", then if you
# specify prefix "./UserFiles/" and suffix "/Kill", the above output will be

#	./UserFiles/abc/Kill ./UserFiles/def/Kill ./UserFiles/ghi/Kill 

# Pass that to "cat", and you get the killfiles for those three accounts.

unshift (@INC,"Scripts");		# get "require" files from Scripts directory
require "header.ph";			# for $Header1 and $Header2

# --

if (@ARGV == 0) {
	$prefix = "";
	$suffix = " ";
} elsif (@ARGV == 1) {
	$prefix = $ARGV[0];
	$suffix = " ";
} elsif (@ARGV == 2) {
	$prefix = $ARGV[0];
	$suffix = $ARGV[1]." ";
} else {
	print STDERR "usage: perl $0 [prefix [suffix]]\n",
				 "    extract the first column of tab-delimited input, collapsing it into one line\n",
				 "    the optional prefix is added before each field;";
				 "    the suffix is added after each field\n";
	exit (1);
}

# --

$out = "";

while ($line = <STDIN>) {
	next if ($line eq $Header1 || $line eq $Header2);			# skip the header lines
	$line =~ /^([^\t]+)/ && ($out .= $prefix.$1.$suffix);
}

print $out, "\n";
