#!/usr/bin/perl

# "config.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College


# ----- version

# This version string is inserted at the end of each user's news home page.
# You might use it to include, say, a link to a page of instructions.

$GatewayVersion = "<HR>Forum News Gateway 0.5.1\012";

# ----- servers

# List the NNTP servers that will be accessible by users.
# In the form "host.domain.edu:port", but the port can be omitted if it
# is 119 (the standard NNTP port).  The first server in the list
# is considered the default server, and any URL with no server specified
# in it (this includes news:group.name and news:[msgid] URLs) will be
# assumed to refer to the default server. Servers which do not exist
# (maybe you mistyped) are noticed on startup and ignored; if the first
# one is ignored then the next one becomes the default, and so on.

@NNTPServers = ("forum.swarthmore.edu","larch.cc.swarthmore.edu");

# You can give the news servers more readable names for the benefit of
# the non-nerd user.
# For each entry in @NNTPServers, write an alias in @NNTPServerAliases.
# If the list of aliases is empty, users see the bare machine names
# from @NNTPServers.

@NNTPServerAliases = ();

# Example:
# @NNTPServerAliases = ("Geometry Forum Server","Swarthmore College Server");

# How long should an NNTP connection be allowed to linger without being
# closed and re-opened? A connection is considered "stale" if it is older
# than $NNTPConnectionSeconds seconds. Stale connections are closed, and
# their associated data is flushed to disk, protecting it in case of a crash.

# There's at least one NNTP server out there which will not tell the gateway
# about newly-arrived articles unless the gateway closes and re-opens the
# NNTP connection. If you find that news home pages insist that all subscribed
# groups are empty when you know that new articles have arrived, this may be
# the problem. Set $NNTPConnectionSeconds to some small value, such as fifteen
# minutes (15*60 seconds).

$NNTPConnectionSeconds = 2*60*60;


# ----- bug workarounds

# -- Fetch articles by number.

# The problem:
# Some defective NNTP servers cannot fetch articles by article ID. Painful
# but true. If you suffer from one of these mush-brain mutants, set
# $FetchArticlesByNumber = 1 to fetch articles by article number instead.
# The setting affects all news servers.

# The symptoms:
# If, when trying to read articles, you get systematic "Your request could
# not be satisfied" messages, and the message mentions that the news server
# returned code 430, then this is probably the difficulty. For some news
# servers, you may get such errors only in certain newsgroup hierarchies.

# It is an option because I fear that somewhere there is another broken
# server which doesn't understand article numbers. Paranoia? You tell me.

$FetchArticlesByNumber = 0;


# -- Turn off SIGPIPE handling.

# The problem:
# The gateway normally relies on a SIGPIPE signal to tell it when
# the client web browser has closed the connection, breaking the
# gateway's output pipe. Unfortunately, on some systems the gateway
# is unable to return from handling a SIGPIPE signal, and crashes
# instead. The problem seems to be a bug or incompatibility in either
# perl or the operating system, I don't know which.

# The symptoms:
# You may be suffering this problem if, whenever you interrupt your
# browser while it is loading a gateway page, the gateway crashes
# with the message
# 	"Probably, the client closed the network connection early. (1)"
# The message is nothing to worry about; it should be common. Worry
# about the crashes.

# Set $IgnoreSIGPIPE = 1 if you suspect this problem. Leave it alone
# otherwise; I fear that ignoring SIGPIPE when you don't have to
# could cause problems on some systems.

$IgnoreSIGPIPE = 0;


# ----- use XOVER NNTP command

# Some NNTP servers support the XOVER command for fetching article
# overview information. XOVER is much faster than the alternative
# of repeated HEAD and NEXT commands. Unfortunately, on at least one
# server the XOVER command is broken in that it will return information
# about articles which do not exist.

# If $UseXOVER is 1, the gateway will attempt to use XOVER. If
# this fails, it will fall back on HEAD and NEXT. The overhead of
# trying XOVER when it is not supported is usually slight.
# If $UseXOVER is 0, the gateway always uses HEAD and NEXT.

# Leave $UseXOVER set to 1 unless either:
# (1) You know that none of your NNTP servers uses XOVER, and
#     don't expect them to in the future, or
# (2) you suspect that XOVER may be broken on one of your NNTP servers.

# A possible symptom of broken XOVER is that newsgroups frequently
# contain articles which, when you click on them, turn out not to exist.
# Unfortunately, other problems can cause this symptom too.

$UseXOVER = 1;

# ----- group filter pattern

# Newsgroups whose names match the group filter pattern, a perl regular
# expression, are not accessible to users. If the pattern is "", then all
# groups are allowed.

$GroupFilterPattern = "";

# Example. Disallow access to local groups with names starting "_":
# $GroupFilterPattern = "^_";


# ----- working files

# The following variables relate to where various data files, such as
# newsrc's, killfiles, threadDB's and allgroups files, are stored.

# By default, all the data files are stored relative to the current directory.
# NOTE: If you change this to something like
#	$WorkingDirectory = "/some/directory";
# then don't forget to remove the chop() line just below!
# Also, if you change $WorkingDirectory, make sure that the .html files
# currently in "./HTMLFiles" can still be found. You can do that by either
# moving the HTMLFiles directory into the new working directory, or by
# making sure that $HTMLFilesRoot, below, still points to the right place.
$WorkingDirectory = `pwd`;
chop($WorkingDirectory);

$PublicDataFilesRoot = $WorkingDirectory."/PublicFiles";

# The directory which contains html files to be served.
# Most pages served are generated on the fly, but some are
# looked up in this directory.

$HTMLFilesRoot = $WorkingDirectory."/HTMLFiles";

# Newsrc's and killfiles go in here, in a subdirectory with the same name
# as the user.
$UserFilesDirectoryName = $WorkingDirectory."/UserFiles";

$ThreadDBFilesDirectoryName = $PublicDataFilesRoot."/ThreadDBs";
$AllGroupsFilesDirectoryName = $PublicDataFilesRoot."/AllGroups";


# ----- account creation and site authorization

# If users are to be able to create their own accounts, set
# $UserAccountCreation = 1. Anyone who is able to connect to the gateway
# will be able to create an account on it (possibly lying about their
# name and e-mail address and so on).

# If the administrator is to create all accounts, set
# $UserAccountCreation = 0.

# The administrator can always create accounts, whether users are allowed
# to or not.

$UserAccountCreation = 1;

# SiteAuthorized()
# This routine should return 1 if users on the connecting host are to be
# allowed to use the gateway; otherwise 0. If your site is not authorized,
# all you see is the site-not-authorized html page.
# By default, this always returns 1. Change it if you want to restrict usage.

# Here are two globals that the routine can refer to:
# $RemoteIPNum    - the IP number of the connecting machine
#                   (as a four-character string, in the usual Perl way)
# $RemoteHostName - a string, the name of the connecting machine
# The remote host name will look like "forum.swarthmore.edu" if we are able
# to get the machine's name; otherwise it will look like "130.58.72.2".

sub SiteAuthorized {
	return 1;
}

# Example. Restrict access to users connecting from 130.58.72.2:

#sub SiteAuthorized {
#	return $RemoteIPNum eq pack('C4',130,58,72,2);
#}

# Example. Restrict access to users in domain ".swarthmore.edu"
# whose machine name we can get:

#sub SiteAuthorized {
#	return $RemoteHostName =~ /\.swarthmore\.edu$/;
#}


# ----- page expiration

# The HTTP protocol provides an expiration time for HTML pages. Some web
# clients even pay attention to it: when a page expires it is purged from
# the client's cache, and refetched if it is again needed.

# These variables determine how long different pages take to expire. The
# default settings work fairly well; there's usually no need to change them.
# If you think users should have to explicitly reload expired pages, then
# make all the numbers big; it may cut the load on the server.

# The expiration times are in seconds. An expiration time of 0 should mean
# that the page expires immediately and should not be cached. Since most
# clients ignore the zero expirations, I've tried an expiration time of
# 1 second; maybe they'll pay attention to that.
# 60 seconds = one minute
# 60*60 seconds = one hour
# 24*60*60 seconds = 24 hours = one day

# Files, for /WelcomePage and /Files URLs.
$FileExpirationSeconds = 24*60*60;

# For /Administration, /AccountCreation, /Newsreading/PrefsPages,
# and /Newsreading/PostArticlePages URLs (which hardly ever change).
$BuiltInExpirationSeconds = 24*60*60;

# For the home news page, /Newsreading/HomeNewsPage.
$HomeExpirationSeconds = 1;

# For the highlights and kills page, /Newsreading/KillPages.
$KillExpirationSeconds = 1;

# For the newsgroup listings, /Newsreading/AllGroupsPages/.
$AllGroupsExpirationSeconds = 30*60;

# For a newsgroup's article listing, /Newsreading/NewsGroupPages, and
# the list of articles in a thread, /Newsreading/ThreadPages.
# This should be no longer than $HomeExpirationSeconds, or else going
# to a newsgroup page from the news home page may show an out-of-date
# newsgroup page if new articles have arrived. Alas.
$NewsgroupExpirationSeconds = 1;

# For articles proper, /Newsreading/ArticlePages and
# /Newsreading/TextArticlePages.
$ArticleExpirationSeconds = 60*60;


# ----- personal information restrictions

# User names must be at least this long, in characters.

$MinUserNameLength = 2;

# User passwords must be at least this long, in characters.

$MinPasswordLength = 4;

# User e-mail addresses must match this regular expression to be
# considered valid. The default pattern is simpleminded; it requires only
# that the putative e-mail address contain a "@" character. If you set the
# pattern to "", then users are not required to fill in their e-mail
# addresses, even to post articles (any bad effects of that are your
# responsibility :-).

$EmailValidityPattern = "\@";

# Here's a slightly more restrictive pattern, which requires characters
# before and after the @:

# $EmailValidityPattern = "[^\@]+\@[^\@]+";


# ----- serving different types of files via the /Files/... URL.

# To serve a file that a client requests via a /Files URL, the gateway
# needs to know what "Content-type:" header to give it. The content-type
# is a MIME type as defined in RFC 1521,
# <http://ds.internic.net/rfc/rfc1521.txt>.
# The gateway chooses a content-type based on the filename's suffix
# by calling the routine below, TranslateSuffixToType.

# This routine accepts a file name and returns a "Content-type:"
# header value. You can extend it to recognize more file suffixes
# by adding lines of the form
#	return "this/that" if ($Filename =~ /\.xyz$/i);
# If the file does not have a recognized suffix, it is assumed to
# be an HTML file. (So for simplicity the ".html" suffix is not
# recognized.)

sub TranslateSuffixToType {
	local($Filename) = @_;
	
	return "image/gif"  if ($Filename =~ /\.gif$/i);
	return "image/jpeg" if ($Filename =~ /\.jpg$/i || $Filename =~ /\.jpeg$/i);
	return "text/plain" if ($Filename =~ /\.txt$/i || $Filename =~ /\.text$/i);
	
	return "text/html";
}


# ----- logging

# The server outputs log information to standard error. By default, it logs
# errors and a few other tidbits. You can choose to log more information.

# To log changes of news server, news group, and user,
# set $LogChanges = 1.

$LogChanges = 0;

# To log the address of each connecting machine at each request,
# set $LogSites = 1.
# To log the URL of each request, set $LogURLs = 1.
# If you set them both to 1, you'll be able to see what URL each
# site requested.

$LogSites = 0;
$LogURLs = 0;

# To log unauthorized accesses, set $LogUnauth = 1.
# Both unauthorized sites and users who mistype their name or password
# will appear in the log, with site and URL information (but not password,
# of course).

$LogUnauth = 0;

# To log that a response has been sent to a request, set $LogReply = 1.
# This is handy for debugging.

$LogReply = 0;

1;
