#!/usr/bin/perl

# "httpdlib.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# ReadHTTPRequest()
# reads the request line, header lines, and body from the CLIENT filehandle.
# returns ($Method,$Url,$ProtoVers,$Headers,$Body), which are all strings.

sub ReadHTTPRequest {
	local($Method) = "";
	local($Url) = "";
	local($ProtoVers) = "";
	local($Headers) = "";
	local($Body) = "";
	local($NextLine) = "";
	
	if ($NextLine = <CLIENT>) {
#		print STDERR "Got request line: \"$NextLine\"";
		if ($NextLine =~ /^\s*(\S+)\s+([^\s\?]+)(\?([\S]*))?/) {
			# We have at least a valid HTTP/0.9 request line
			$Method = $1;
			$Url = &URLDecode($2,0);
			$Body = $4;
			if ($NextLine =~ /^\s*(\S+)\s+([^\s\?]+)(\?([\S]*))?\s+([\S]+)/) {
				# Now we know we have a valid HTTP/1.0 request line
				$ProtoVers = $5;

				local($ContentLength) = -1;
				# Now look for any headers following the request line
				while (($NextLine = <CLIENT>) && ($NextLine ne "\015\012")) {
#					print STDERR "Got header line \"$NextLine\"";
					if ($NextLine =~ /^content\-length:\s*(\d+)/i) {
						$ContentLength = $1;
					}
					$Headers .= $NextLine;
				}

#				if ($NextLine eq "\015\012") {
#					print STDERR "Got CRLF after headers\n";
#				}
				# now get the body if there is one
				if (($Method eq "POST") || ($Method eq "PUT")) {
					if ($ContentLength > 0) {
						# Read $ContentLength bytes
#						print STDERR "About to do the dreaded body read.  \$ContentLength is $ContentLength.\n";
						# I hate to do the following getc stuff, but I can't get read to work. - Asa
						for ( ; $ContentLength > 0; $ContentLength-- ) {
#							local($NextChar);
#							read(CLIENT,$NextChar,1);
							local($NextChar) = getc(CLIENT);
							if ($NextChar eq "") {
								print STDERR "In ReadHTTPRequest: Premature end of input reading binary body.\n";
								last;
							}
							$Body .= $NextChar;
						}
#						read(CLIENT,$Body,$ContentLength) || print STDERR "In ReadHTTPRequest: read of binary body failed: $!";
#						print STDERR "read completed\n";
					} elsif ($ContentLength < 0) {
						# read lines until eof
						while ($NextLine = <CLIENT>) {
#							print STDERR "Got body line \"$NextLine\"\n";
							$Body .= $NextLine;
						}
					}
				}
			}
		}
	}
	return ($Method, $Url, $ProtoVers, $Headers, $Body);
}

# ParseURL($Url)
# Looks at a URL to see if it's one of the ones we recognize. If so, breaks
# it down into a command part and an arguments part and returns the part of the
# URL after the hostname and port, broken into pieces at the slashes.
# If not, returns ().

sub ParseURL {
	local($Url) = @_;
	local(@Result) = ();
	if ($Url =~ m|^(http:)?(//(([\w\.]+)(:(\d+))?)?)?/(.*)(/)?|) {
		@Result = split(/\//,$7);
	} elsif ($Url =~ /^news:(.*)/) {
		local($AfterNewsPart) = $1;
		if ($AfterNewsPart eq "") {
			# Then it's the news home page
			@Result = ("Newsreading", "HomeNewsPage");
		} else {
			$AfterNewsPart =~ m|(//(([\w\.]+)(:(\d+))?)?)?(/)?(.*)|;
			local($Server) = $2;
			local($Command) = $7;
			if ($Command =~ /\@/) {
				# Then it's a message-id.
				@Result = ("Newsreading","ArticlePages",$Server,"",$Command);
			} elsif ($Command =~ /(^|\.)\*/) {
				# Then it's a pattern like * or alt.*
				local(@GroupPrefixParts) = split(/\./,$Command);
				pop(@GroupPrefixParts);
				@Result = ("Newsreading","AllGroupsPages",$Server,@GroupPrefixParts);
			} else {
				# Then it's a groupname
				@Result = ("Newsreading","NewsGroupPages",$Server,$Command);
			}
		}
#		print STDERR "received news: URL - \"$Url\"\n";
#		print STDERR "converted it to http: URL - \"",join('/',@Result),"\"\n";
	}
	
	return @Result;
}

# BuildHeaderArray($Headers)
# Builds an associative array of (headername,values) pairs.  If a given header
# appears more than once, the two values will be stuck together with "\0"'s.

sub BuildHeaderArray {
	local($Headers) = @_;
	local(@HeaderLines) = split(/\015\012/,$Headers);
	local(%Result) = ();
	local($HeaderLine);
	local($FieldName) = "";
	local($FieldValue) = "";
	
	foreach $HeaderLine (@HeaderLines) {
		if ($HeaderLine =~ /([\S]+):\s*([^\015\012]*)/) {
			$FieldName = $1;
			$FieldValue = $2;
			$FieldName =~ tr/A-Z/a-z/;
#			print STDERR "Parsing header: \"$FieldName\" = \"$FieldValue\"\n";
			if ($Result{$FieldName} ne "") {
				$Result{$FieldName} .= "\0";
			}
			$Result{$FieldName} .= $FieldValue;
		}
	}
	
	return %Result;
}

# BuildFormValuesArray($Body)
# This routine takes the body of an object of MIME type application/x-www-form-urlencoded
# and turns it into an associative array of (fieldname,value) pairs.

sub BuildFormValuesArray {
	local($Body) = @_;
	local(@BodyLines) = split(/\&/,$Body);
	local(%Result) = ();
	local($BodyLine,$Key,$Value);
	
	foreach $BodyLine (@BodyLines) {
		if ($BodyLine =~ /([^=]+)=([^=]*)/) {
			$Key = &URLDecode($1,1);
			$Value = &URLDecode($2,1);
#			print STDERR "Parsing form Value: \"$Key\"=\"$Value\"\n";
			$Result{$Key} = $Value;
		}
	}
	
	return %Result;
}

# IsAuthorized($AuthHeader,$FirstURLPart)
# This routine takes the content of the Authorization: header of an HTTP
# request and checks if it is valid.  If so, it sets the current user
# appropriately.  Returns nonzero if the authentication succeeded, zero
# if it didn't.  The $FirstURLPart determines which users are allowed access.

sub IsAuthorized {
	local($AuthHeader,$FirstURLPart) = @_;
	
	if ($FirstURLPart eq "WelcomePage" ||
		$FirstURLPart eq "AccountCreationPage" ||
		$FirstURLPart eq "Files") {
		return 1;
	}
	
	# These two lines disable authorization so we can test with browsers that don't support it.
#	&SetCurrentUser("packer");
#	return 1;
	
	if ($AuthHeader =~ /Basic\s+(\S+)/i) {
		local($NameAndPassword) = &Base64Decode($1);
		if ($NameAndPassword =~ /([^:]+):([^:]+)/) {
			local($Username,$Password) = ($1,$2);
			if (&DoesUserExist($Username)) {
				&SetCurrentUser($Username);
				if (&GuessPassword($Password,$Newsrc{"prefs","password"})) {
					if ($FirstURLPart eq "Administration" && $Username ne "root") {
						&SetCurrentUser("");
						return 0; 	# in your dreams, chump!
					} else {
						return 1;
					}
				} else {
					&SetCurrentUser("");
					return 0;
				}
			}
		}
	}
	
	return 0;
}

# SendToClient(@StuffToSend)

sub SendToClient {
	(print CLIENT @_) || &Fail($ERR_CLIENT_DIED,"Probably, the client closed the network connection early. (2)");
}

# SendNNTPCommand($Command)
# Returns ($ResponseCode,$ResponseText).

sub SendNNTPCommand {
	local($Command) = @_;
	
#	print STDERR " -- $CurrentNNTPServer $Command";
	
	# If the NNTP connection broke, then the print command here may raise a
	# SIGPIPE signal. Set this flag so that PipeSigHandler will know that it
	# was the NNTP pipe that broke, not the client pipe.
	$NNTPCommandInProgress = 1;
	print NNTPSERVER $Command;
	$NNTPCommandInProgress = 0;
	
	local($ResponseCode,$ResponseText) = &ReadNNTPResponseLine();
	if (!$RecoveringFromTimeout && ($ResponseCode == 503 || $ResponseCode == 400)) {
		$RecoveringFromTimeout = 1;
		# Server timed out or discontinued us - let's try to reconnect.
		print STDERR "NNTP server $CurrentNNTPServer disconnected with code $ResponseCode - attempting to reestablish connection\n";
		local($Server) = $CurrentNNTPServer;
		local($AllGroupsLoaded) = $CurrentNNTPServerAllGroupsLoaded;
		local($Group) = $CurrentGroup;
		local($ThreadDBLoaded) = $CurrentGroupThreadDBLoaded;
		&SetCurrentNNTPServer("",0);
		&SetCurrentNNTPServer($Server,$AllGroupsLoaded);
		&SetCurrentGroup($Group,$ThreadDBLoaded);
		print STDERR "Successfully reestablished connection.\n";
		$NNTPCommandInProgress = 1;
		print NNTPSERVER $Command;
		$NNTPCommandInProgress = 0;
		($ResponseCode,$ResponseText) = &ReadNNTPResponseLine();
		$RecoveringFromTimeout = 0;
	}
	
	($ResponseCode,$ResponseText);
}

# ReadNNTPResponseLine()
# This reads an NNTP-style response line with a numeric code and a textual
# response.  It reads from the global filehandle NNTPSERVER.  It returns the
# numeric code and the text response in separate variables.

sub ReadNNTPResponseLine {
	local($NextLine) = "";
	local($ResponseCode) = "";
	local($ResponseText) = "";

	while (($NextLine = <NNTPSERVER>) && ($NextLine =~ /^\d\d\d-/)) {
		# do nothing
	}
	if ($NextLine =~ /^(\d\d\d)\s*([^\015\012]*)/) {
		$ResponseCode = $1;
		$ResponseText = $2;
	}
	
#	print STDERR "NNTP Server response: \"$ResponseCode $ResponseText\"\n";
	
	return ($ResponseCode,$ResponseText);
}

# ReadDotTerminatedText()
# Read NNTP-style dot-terminated text from NNTPSERVER.
# Return an array of lines of text with the newlines still on them.

sub ReadDotTerminatedText {
	local(@Result) = ();
	local($NextLine) = "";

	while (($NextLine = <NNTPSERVER>) && ($NextLine ne ".\015\012")) {
		$NextLine =~ s/^\.\././;
		push(@Result,$NextLine);
	}
	return @Result;
}


# NewsrcKeyCompare
# This function compares things of the form "forum.swarthmore.edu geometry.forum" or
# "larch.cc.swarthmore.edu sci.math".  The comparison actually ignores the
# server names and just does the same thing as NewsgroupNameCompare on the groupnames.
# I can't actually call NewsgroupNameCompare since Perl sort routines are weird.

sub NewsrcKeyCompare {
	local($Group1GroupName,$Group2GroupName);
	if ($a =~ /\S+\s+(\S*)/) {
		$Group1GroupName = $1;
	}
	if ($b =~ /\S+\s+(\S*)/) {
		$Group2GroupName = $1;
	}
	
	local(@Group1Parts) = split(/\./,$Group1GroupName);
	local(@Group2Parts) = split(/\./,$Group2GroupName);
	
	local($Group1Part);
	local($Group2Part);
	local($Result);
	while ((@Group1Parts > 0) && (@Group2Parts > 0)) {
		$Group1Part = shift(@Group1Parts);
		$Group2Part = shift(@Group2Parts);
		
		$Result = $Group1Part cmp $Group2Part;
		return $Result if ($Result);
	}
	
	return (@Group1Parts <=> @Group2Parts);
}

# NewsgroupNameCompare
# This routine can be used by the sort function to sort an array of newsgroup
# names.  The function compares parts of the groupnames at a time, rather than
# letters at a time like the standard lexicographical ordering.

sub NewsgroupNameCompare {
	local(@Group1Parts) = split(/\./,$a);
	local(@Group2Parts) = split(/\./,$b);
	
	local($Group1Part);
	local($Group2Part);
	local($Result);
	while ((@Group1Parts > 0) && (@Group2Parts > 0)) {
		$Group1Part = shift(@Group1Parts);
		$Group2Part = shift(@Group2Parts);
		
		$Result = $Group1Part cmp $Group2Part;
		return $Result if ($Result);
	}
	
	return (@Group1Parts <=> @Group2Parts);
}

# GetNNTPHostFromURL(@UrlParts)
# This routine looks at the second part of the URL and for a hostname and port.
# If the second part of the URL is empty then this routine returns the default host and port.

sub GetNNTPHostFromURL {
	local(@UrlParts) = @_;
	
	if ($UrlParts[2] eq "") {
		return $CanonicalNNTPServers[0];
	} else {
		return $UrlParts[2];
	}
}

# UpdateAllGroupsTree()
# Gets the groups that have been created since the last update (or all groups if
# this is the first update), and inserts them into the tree.  The tree is an associative
# array of (folder,subfolders) pairs.
# subfolders is a string beginning with a slash and then containing all the
# subfolders delimited by slashes.  So to get the subfolders, split at the slashes and
# then discard the first element (which will always be an empty string).
# The only exception to this is $AllGroups{""}, which also has the time of last update as
# shown below.
# Here are some examples of how the array works:
# $AllGroups{""} is "940401 235959 GMT/alt/biz/control/comp/junk/rec/talk/sci/soc" (all the top-level folders, with the date of last update at the beginning)
# $AllGroups{"sci"} is "/math/physics/chem/bio" (etc.)
# $AllGroups{"comp.sys.mac.programmer"} is "/" (because comp.sys.mac.programmer is a group; it has no subfolders)
# $AllGroups{"alt.sex"} is "/movies//stories" (note the empty string stuck between the two slashes - because
#															   alt.sex is a group as well as a parent heading)

sub UpdateAllGroupsTree {
	local(@NewGroups);
	local(@DateList) = split('/',$AllGroups{""},2);
	local($DateLastUpdated) = shift(@DateList);
	local($DateThisUpdate) = &TimeString(time);
	if ($DateLastUpdated ne "") {
		if ($DateLastUpdated =~ /^\d{6}\s+\d{6}/) {
			@NewGroups = &GetListOfNewGroups($DateLastUpdated);
#			print STDERR "Updating all groups array in memory...\n";
		} else {
			print STDERR "In UpdateAllGroupsTree: Date of last update for all groups tree is weird: \"$DateLastUpdated\"\n";
			return;
		}
	} else {
		@NewGroups = &GetListOfAllGroups();
#		print STDERR "Building all groups array in memory...\n";
	}
	
	# Add each new group to the array
	local($Group,$GroupName,@GroupNameParts,$Prefix,$Part);
	foreach $Group (@NewGroups) {
		$Group =~ /^(\S+)/;									# the whole line, starting with group name
		$GroupName = $1;									# the group name only
		next if ($GroupFilterPattern ne "" && $GroupName =~ /$GroupFilterPattern/o);	# skip if filtered out
#		print STDERR "Adding group \"$GroupName\"...\n";
		@GroupNameParts = split(/\./,$GroupName);			# the segments of the group name
		$Prefix = "";
		$Part = "";
		foreach $Part (@GroupNameParts) {
			# add $Part to the list of subfolders of $Prefix, if it's not already there
#			print STDERR "\$Part is $Part\n";
			local($Subfolders) = $AllGroups{$Prefix};
			local($PartPattern) = $Part;
			$PartPattern =~ s/(\W)/\\$1/g;
			($Subfolders !~ m#/$PartPattern(/|$)#) && ($AllGroups{$Prefix} .= '/'.$Part);
#			print STDERR "\$AllGroups\{$Prefix\} is $AllGroups{$Prefix}\n";
			($Prefix ne "") && ($Prefix .= '.');
			$Prefix .= $Part;
		}
		# $Prefix now contains the whole groupname, and we represent a group in the array by
		# an entry with a subfolder named the empty string.
		($AllGroups{$Prefix} !~ m#/(/|$)#) && ($AllGroups{$Prefix} .= '/');
	}
	
	# Change time of last update for all groups list
	local(@Top) = split('/',$AllGroups{""},2);
	shift(@Top);
	unshift(@Top,&TimeStringToNNTPTimeString($DateThisUpdate));
	$AllGroups{""} = join('/',@Top);
}

# URLDecode($EncodedString,$RecognizePlusses)
# This routine decodes a string that has been encoded using the URL scheme
# for representing funky characters

sub URLDecode {
	local($String) = @_;
	
	# Replace '+' with ' '.
	$String =~ s/\+/ /g;
	
	# Replace %xx (% followed by two hex digits) with the character
	# encoded by the hex digits.
	$String =~ s/%(..)/pack("C",hex($1))/eg;
	
	return $String;
}

# Base64Decode($EncodedString)
# Decode a base64-encoded string into plain text.
# This routine was written for speed, but it is still slow.
# This routine assumes that $[ is 0, which is correct for the Perly Gateway.

# This array is global so that it doesn't have to be rebuilt on each call.
%Decoding = ('A',0x00,'B',0x01,'C',0x02,'D',0x03,'E',0x04,'F',0x05,'G',0x06,'H',0x07,
			 'I',0x08,'J',0x09,'K',0x0A,'L',0x0B,'M',0x0C,'N',0x0D,'O',0x0E,'P',0x0F,
			 'Q',0x10,'R',0x11,'S',0x12,'T',0x13,'U',0x14,'V',0x15,'W',0x16,'X',0x17,
			 'Y',0x18,'Z',0x19,'a',0x1A,'b',0x1B,'c',0x1C,'d',0x1D,'e',0x1E,'f',0x1F,
			 'g',0x20,'h',0x21,'i',0x22,'j',0x23,'k',0x24,'l',0x25,'m',0x26,'n',0x27,
			 'o',0x28,'p',0x29,'q',0x2A,'r',0x2B,'s',0x2C,'t',0x2D,'u',0x2E,'v',0x2F,
			 'w',0x30,'x',0x31,'y',0x32,'z',0x33,'0',0x34,'1',0x35,'2',0x36,'3',0x37,
			 '4',0x38,'5',0x39,'6',0x3A,'7',0x3B,'8',0x3C,'9',0x3D,'+',0x3E,'/',0x3F);

sub Base64Decode {
	local($EncodedString) = @_;
	local($Length) = length($EncodedString);
	local($DecodedString) = "";
	
	local($Char1,$Char2,$Char3,$Char4);
	
	# Do all code groups except the final one.
	while ($Length > 4) {
		$Char1 = substr($EncodedString,0,1);
		$Char2 = substr($EncodedString,1,1);
		$Char3 = substr($EncodedString,2,1);
		$Char4 = substr($EncodedString,3,1);
		substr($EncodedString,0,4) = "";
		$Length -= 4;
		
		$DecodedString .= pack("C3",(($Decoding{$Char1} & 0x3F) << 2) | (($Decoding{$Char2} & 0x30) >> 4),
									(($Decoding{$Char2} & 0x0F) << 4) | (($Decoding{$Char3} & 0x3C) >> 2),
									(($Decoding{$Char3} & 0x03) << 6) |  ($Decoding{$Char4} & 0x3F));
	}
	
	# Do the final code group.
	if ($Length >= 2) {
		# Pushing these assignments farther into the if's does not measurably improve speed.
		$Char1 = substr($EncodedString,0,1);
		$Char2 = substr($EncodedString,1,1);
		$Char3 = substr($EncodedString,2,1);
		$Char4 = substr($EncodedString,3,1);
 		if ($Char1 ne "=" && $Char2 ne "=") {
 			$DecodedString .= pack("C",(($Decoding{$Char1} & 0x3F) << 2) | (($Decoding{$Char2} & 0x30) >> 4));
			
			if ($Length >= 3 && $Char3 ne "=") {
				$DecodedString .= pack("C",(($Decoding{$Char2} & 0x0F) << 4) | (($Decoding{$Char3} & 0x3C) >> 2));
				
				if ($Length >= 4 && $Char4 ne "=") {
					$DecodedString .= pack("C",(($Decoding{$Char3} & 0x03) << 6) | ($Decoding{$Char4} & 0x3F));
				}
			}
		}
	}
	
	return $DecodedString;
}

# NoDieCrypt($String)
# Calls crypt, wrapped in an eval, so if we're on a Mac we won't die.

sub NoDieCrypt {
	local($String) = @_;
	srand(time|$$);
	local($SaltInt) = int(rand(4096));
	local($SaltChars) = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./';
	local($Salt) = substr($SaltChars,(($SaltInt & 4032)/64),1).substr($SaltChars,($SaltInt % 64),1);
	eval '$String = crypt($String,$Salt)';
	
	$String;
}

# GuessPassword($Guess,$EncryptedPassword)
# Returns non-zero if and only if the guess is correct.
# On a Mac, this routine degenerates to a simple string comparison.

sub GuessPassword {
	local($Guess,$EncryptedPassword) = @_;
	
	if ($EncryptedPassword eq "") {
		# Then probably we're trying to guess the password of a user who doesn't exist, so fail.
		return 0;
	}
	
	local($Result) = $Guess eq $EncryptedPassword; # In case the eval below dies (i.e., we're on a Mac).
	eval '$Result = crypt($Guess,$EncryptedPassword) eq $EncryptedPassword';
	
	$Result;
}

# HTMLSafeify($String)
# This routine replaces characters that have significance in HTML, like <, >, and &,
# with escape codes.

sub HTMLSafeify {
	local($String) = @_;
	
	$String =~ s/\&/&amp;/g;
	$String =~ s/</&lt;/g;
	$String =~ s/>/&gt;/g;
	
	$String;
}

sub PossessifyName {
	local($Name) = @_;
	
	if ($Name =~ /s$/i) {
		$Name .= "'";
	} else {
		$Name .= "'s";
	}
	
	$Name;
}

# Fail($ExceptionCode,$ExceptionText)
# This is called when an operation fails. It sets the global error code and
# error message and dies. It should be called only within an eval, so the
# failure won't kill the whole server, only the operation in progress.
# Negative exception codes are defined in "globals.pl" &InitGlobalVars.
# Positive exception codes are response codes from the NNTP server.

sub Fail {
	($ExceptionCode,$ExceptionText) = @_;
	print STDERR "Failing! Code=$ExceptionCode, Text=\"$ExceptionText\"\n";
	die "";
}

# SendProtoVers($ProtoVers,$ExpirationSeconds)
# If $ProtoVers (protocol version) is given, send default protocol version
# and other header information. The expiration time is now+$ExpirationSeconds.

sub SendProtoVers {
	local($ProtoVers,$ExpirationSeconds) = @_;
	
	if ($ProtoVers ne "") {
		local($Now) = time;
		&SendToClient(	"HTTP/1.0 200 OK\012",
				  		"Date: ",&TimeString($Now),"\012",
						"Expires: ",&TimeString($Now+$ExpirationSeconds),"\012",
						"Mime-version: 1.0\012",
						"Content-type: text/html\012\012");
	}
}

# SendFileToClient($Url,$ProtoVers,$FileName)
# Look up the given file in the $HTMLFilesRoot directory and send its
# contents of the given file to the client. If $FileName (actually a pathname)
# contains the characters ".." anywhere within it, we consider that it may be
# an attempt to break security by returning a file above $HTMLFilesRoot.
# This means you can't have any legitimate files with ".." in their names.
# This routine sends out the root page and any other $HTMLFilesRoot pages.
# It calls TranslateSuffixToType in "config.pl" to translate the file suffix
# into a content-type header value.

sub SendFileToClient {
	local($Url,$ProtoVers,$Filename) = @_;
	local($Buffer);
	
#	print STDERR "Sending file ", $HTMLFilesRoot."/".$Filename, "\n";
	
	if ($Filename =~ /\.\./ || !open(INFILE,"<".$HTMLFilesRoot."/".$Filename)) {
		&SendHTTPNotFoundMessage($Url,$ProtoVers);
		return;
	}
	
	if ($ProtoVers ne "") {
		local($Now) = time;
		&SendToClient(	"HTTP/1.0 200 OK\012",
						"Date: ",&TimeString($Now),"\012",
						"Expires: ",&TimeString($Now+$FileExpirationSeconds),"\012",
						"Mime-version: 1.0\012",
						"Content-type: ",&TranslateSuffixToType($Filename),"\012\012");
	}
	
	binmode INFILE;
	while (read(INFILE,$Buffer,32768)) {
		(print CLIENT $Buffer) || &Fail($ERR_CLIENT_DIED,"Client disconnected while receiving file.");
	}
	
	close(INFILE);
}

1;
